import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:yona_crypto/b64.dart';
import 'package:yona_crypto/crypto.dart';

class Msg {
  final B64 from;

  final String data;
  final B64 to;
  final String? id;

  @override
  String toString() {
    Map<String, String> params = {
      "data": data,
      "from": from.toString(),
      "to": to.toString(),
    };
    if (id != null) {
      params["id"] = id!;
    }
    return Uri(queryParameters: params).query;
  }

  static Msg fromString(String str, [B64? to]) {
    var params = Uri.splitQueryString(str);
    var to2 = params["to"] ?? to!.toString();
    return Msg(B64(params["from"]!), B64(to2), params["data"]!, params["id"]);
  }

  Msg(this.from, this.to, this.data, [this.id]);
}

Future<void> put(Uri host, Msg msg, B64 signature) async {
  verify(utf8.encoder.convert(msg.data), signature, msg.from);
  var resp = await http.put(host,
      headers: {"Content-Signature": signature.toString()},
      body: msg.toString());
  if (resp.statusCode != 200 && resp.statusCode != 204) {
    throw (resp.body == "" ? resp.statusCode : resp.body);
  }
}

Future<Msg> get(Uri host, B64 to, [String? since = "0"]) async {
  if (since == null || since.isEmpty) since = "0";
  var resp = await http.get(Uri(
      scheme: "http",
      host: host.host,
      port: host.port,
      queryParameters: {"since": since},
      pathSegments: [to.toString()]));
  if (resp.statusCode != 200) {
    throw (resp.body == "" ? resp.statusCode : resp.body);
  }
  return Msg.fromString(resp.body, to);
}
